// Generated from player/lua/stats.lua

"-- Display some stats.\n"
"--\n"
"-- Please consult the readme for information about usage and configuration:\n"
"-- https://github.com/Argon-/mpv-stats\n"
"--\n"
"-- Please note: not every property is always available and therefore not always\n"
"-- visible.\n"
"\n"
"local mp = require 'mp'\n"
"local options = require 'mp.options'\n"
"local utils = require 'mp.utils'\n"
"local input = require 'mp.input'\n"
"\n"
"-- Options\n"
"local o = {\n"
"    -- Default key bindings\n"
"    key_page_1 = \"1\",\n"
"    key_page_2 = \"2\",\n"
"    key_page_3 = \"3\",\n"
"    key_page_4 = \"4\",\n"
"    key_page_5 = \"5\",\n"
"    key_page_0 = \"0\",\n"
"    -- For pages which support scrolling\n"
"    key_scroll_up = \"UP\",\n"
"    key_scroll_down = \"DOWN\",\n"
"    key_search = \"/\",\n"
"    scroll_lines = 1,\n"
"\n"
"    duration = 4,\n"
"    redraw_delay = 1,                -- acts as duration in the toggling case\n"
"    ass_formatting = true,\n"
"    persistent_overlay = false,      -- whether the stats can be overwritten by other output\n"
"    filter_params_max_length = 100,  -- show one filter per line if list exceeds this length\n"
"    file_tag_max_length = 128,       -- only show file tags shorter than this length in bytes\n"
"    file_tag_max_count = 16,         -- only show the first x file tags\n"
"    show_frame_info = false,         -- whether to show the current frame info\n"
"    term_width_limit = -1,           -- overwrites the terminal width\n"
"    term_height_limit = -1,          -- overwrites the terminal height\n"
"    debug = false,\n"
"\n"
"    -- Graph options and style\n"
"    plot_perfdata = true,\n"
"    plot_vsync_ratio = true,\n"
"    plot_vsync_jitter = true,\n"
"    plot_tonemapping_lut = false,\n"
"    skip_frames = 5,\n"
"    global_max = true,\n"
"    flush_graph_data = true,         -- clear data buffers when toggling\n"
"    plot_bg_border_color = \"0000FF\",\n"
"    plot_bg_color = \"262626\",\n"
"    plot_color = \"FFFFFF\",\n"
"    plot_bg_border_width = 0.5,\n"
"\n"
"    -- Text style\n"
"    font = \"\",\n"
"    font_mono = \"monospace\",   -- monospaced digits are sufficient\n"
"    font_size = 8,\n"
"    font_color = \"\",\n"
"    border_size = 0.8,\n"
"    border_color = \"\",\n"
"    shadow_x_offset = 0.0,\n"
"    shadow_y_offset = 0.0,\n"
"    shadow_color = \"\",\n"
"    alpha = \"11\",\n"
"    vidscale = \"auto\",\n"
"\n"
"    -- Custom header for ASS tags to style the text output.\n"
"    -- Specifying this will ignore the text style values above and just\n"
"    -- use this string instead.\n"
"    custom_header = \"\",\n"
"\n"
"    -- Text formatting\n"
"    -- With ASS\n"
"    ass_nl = \"\\\\N\",\n"
"    ass_indent = \"\\\\h\\\\h\\\\h\\\\h\\\\h\",\n"
"    ass_prefix_sep = \"\\\\h\\\\h\",\n"
"    ass_b1 = \"{\\\\b1}\",\n"
"    ass_b0 = \"{\\\\b0}\",\n"
"    ass_it1 = \"{\\\\i1}\",\n"
"    ass_it0 = \"{\\\\i0}\",\n"
"    -- Without ASS\n"
"    no_ass_nl = \"\\n\",\n"
"    no_ass_indent = \"\\t\",\n"
"    no_ass_prefix_sep = \" \",\n"
"    no_ass_b1 = \"\\027[1m\",\n"
"    no_ass_b0 = \"\\027[0m\",\n"
"    no_ass_it1 = \"\\027[3m\",\n"
"    no_ass_it0 = \"\\027[0m\",\n"
"\n"
"    bindlist = \"no\",  -- print page 4 to the terminal on startup and quit mpv\n"
"}\n"
"options.read_options(o)\n"
"\n"
"o.term_width_limit = tonumber(o.term_width_limit) or -1\n"
"o.term_height_limit = tonumber(o.term_height_limit) or -1\n"
"if o.term_width_limit < 0 then\n"
"    o.term_width_limit = nil\n"
"end\n"
"if o.term_height_limit < 0 then\n"
"    o.term_height_limit = nil\n"
"end\n"
"\n"
"local format = string.format\n"
"local max = math.max\n"
"local min = math.min\n"
"\n"
"-- Scaled metrics\n"
"local font_size = o.font_size\n"
"local border_size = o.border_size\n"
"local shadow_x_offset = o.shadow_x_offset\n"
"local shadow_y_offset = o.shadow_y_offset\n"
"local plot_bg_border_width = o.plot_bg_border_width\n"
"-- Function used to record performance data\n"
"local recorder = nil\n"
"-- Timer used for redrawing (toggling) and clearing the screen (oneshot)\n"
"local display_timer = nil\n"
"-- Timer used to update cache stats.\n"
"local cache_recorder_timer\n"
"-- Current page and <page key>:<page function> mappings\n"
"local curr_page = o.key_page_1\n"
"local pages = {}\n"
"local scroll_bound = false\n"
"local searched_text\n"
"local tm_viz_prev = nil\n"
"-- Save these sequences locally as we'll need them a lot\n"
"local ass_start = mp.get_property_osd(\"osd-ass-cc/0\")\n"
"local ass_stop = mp.get_property_osd(\"osd-ass-cc/1\")\n"
"-- Ring buffers for the values used to construct a graph.\n"
"-- .pos denotes the current position, .len the buffer length\n"
"-- .max is the max value in the corresponding buffer\n"
"local vsratio_buf, vsjitter_buf\n"
"local function init_buffers()\n"
"    vsratio_buf = {0, pos = 1, len = 50, max = 0}\n"
"    vsjitter_buf = {0, pos = 1, len = 50, max = 0}\n"
"end\n"
"local cache_ahead_buf, cache_speed_buf\n"
"local perf_buffers = {}\n"
"\n"
"local function graph_add_value(graph, value)\n"
"    graph.pos = (graph.pos % graph.len) + 1\n"
"    graph[graph.pos] = value\n"
"    graph.max = max(graph.max, value)\n"
"end\n"
"\n"
"local function no_ASS(t)\n"
"    if not o.use_ass then\n"
"        return t\n"
"    elseif not o.persistent_overlay then\n"
"        -- mp.osd_message supports ass-escape using osd-ass-cc/{0|1}\n"
"        return ass_stop .. t .. ass_start\n"
"    else\n"
"        return mp.command_native({\"escape-ass\", tostring(t)})\n"
"    end\n"
"end\n"
"\n"
"\n"
"local function bold(t)\n"
"    return o.b1 .. t .. o.b0\n"
"end\n"
"\n"
"\n"
"local function it(t)\n"
"    return o.it1 .. t .. o.it0\n"
"end\n"
"\n"
"\n"
"local function text_style()\n"
"    if not o.use_ass then\n"
"        return \"\"\n"
"    end\n"
"    if o.custom_header and o.custom_header ~= \"\" then\n"
"        return o.custom_header\n"
"    else\n"
"        local style = \"{\\\\r\\\\an7\\\\fs\" .. font_size .. \"\\\\bord\" .. border_size\n"
"\n"
"        if o.font ~= \"\" then\n"
"            style = style .. \"\\\\fn\" .. o.font\n"
"        end\n"
"\n"
"        if o.font_color ~= \"\" then\n"
"            style = style .. \"\\\\1c&H\" .. o.font_color .. \"&\\\\1a&H\" .. o.alpha .. \"&\"\n"
"        end\n"
"\n"
"        if o.border_color ~= \"\" then\n"
"            style = style .. \"\\\\3c&H\" .. o.border_color .. \"&\\\\3a&H\" .. o.alpha .. \"&\"\n"
"        end\n"
"\n"
"        if o.shadow_color ~= \"\" then\n"
"            style = style .. \"\\\\4c&H\" .. o.shadow_color .. \"&\\\\4a&H\" .. o.alpha .. \"&\"\n"
"        end\n"
"\n"
"        return style .. \"\\\\xshad\" .. shadow_x_offset ..\n"
"               \"\\\\yshad\" .. shadow_y_offset .. \"}\"\n"
"    end\n"
"end\n"
"\n"
"\n"
"local function has_vo_window()\n"
"    return mp.get_property_native(\"vo-configured\") and mp.get_property_native(\"video-osd\")\n"
"end\n"
"\n"
"\n"
"-- Generate a graph from the given values.\n"
"-- Returns an ASS formatted vector drawing as string.\n"
"--\n"
"-- values: Array/table of numbers representing the data. Used like a ring buffer\n"
"--         it will get iterated backwards \140len\140 times starting at position \140i\140.\n"
"-- i     : Index of the latest data value in \140values\140.\n"
"-- len   : The length/amount of numbers in \140values\140.\n"
"-- v_max : The maximum number in \140values\140. It is used to scale all data\n"
"--         values to a range of 0 to \140v_max\140.\n"
"-- v_avg : The average number in \140values\140. It is used to try and center graphs\n"
"--         if possible. May be left as nil\n"
"-- scale : A value that will be multiplied with all data values.\n"
"-- x_tics: Horizontal width multiplier for the steps\n"
"local function generate_graph(values, i, len, v_max, v_avg, scale, x_tics)\n"
"    -- Check if at least one value exists\n"
"    if not values[i] then\n"
"        return \"\"\n"
"    end\n"
"\n"
"    local x_max = (len - 1) * x_tics\n"
"    local y_offset = border_size\n"
"    local y_max = font_size * 0.66\n"
"    local x = 0\n"
"\n"
"    if v_max > 0 then\n"
"        -- try and center the graph if possible, but avoid going above \140scale\140\n"
"        if v_avg and v_avg > 0 then\n"
"            scale = min(scale, v_max / (2 * v_avg))\n"
"        end\n"
"        scale = scale * y_max / v_max\n"
"    end  -- else if v_max==0 then all values are 0 and scale doesn't matter\n"
"\n"
"    local s = {format(\"m 0 0 n %f %f l \", x, y_max - scale * values[i])}\n"
"    i = ((i - 2) % len) + 1\n"
"\n"
"    for _ = 1, len - 1 do\n"
"        if values[i] then\n"
"            x = x - x_tics\n"
"            s[#s+1] = format(\"%f %f \", x, y_max - scale * values[i])\n"
"        end\n"
"        i = ((i - 2) % len) + 1\n"
"    end\n"
"\n"
"    s[#s+1] = format(\"%f %f %f %f\", x, y_max, 0, y_max)\n"
"\n"
"    local bg_box = format(\"{\\\\bord%f}{\\\\3c&H%s&}{\\\\1c&H%s&}m 0 %f l %f %f %f 0 0 0\",\n"
"                          plot_bg_border_width, o.plot_bg_border_color, o.plot_bg_color,\n"
"                          y_max, x_max, y_max, x_max)\n"
"    return format(\"%s{\\\\rDefault}{\\\\pbo%f}{\\\\shad0}{\\\\alpha&H00}{\\\\p1}%s{\\\\p0}\" ..\n"
"                  \"{\\\\bord0}{\\\\1c&H%s}{\\\\p1}%s{\\\\p0}%s\",\n"
"                  o.prefix_sep, y_offset, bg_box, o.plot_color, table.concat(s), text_style())\n"
"end\n"
"\n"
"\n"
"local function append(s, str, attr)\n"
"    if not str then\n"
"        return false\n"
"    end\n"
"    attr.prefix_sep = attr.prefix_sep or o.prefix_sep\n"
"    attr.indent = attr.indent or o.indent\n"
"    attr.nl = attr.nl or o.nl\n"
"    attr.suffix = attr.suffix or \"\"\n"
"    attr.prefix = attr.prefix or \"\"\n"
"    attr.no_prefix_markup = attr.no_prefix_markup or false\n"
"    attr.prefix = attr.no_prefix_markup and attr.prefix or bold(attr.prefix)\n"
"\n"
"    local index = #s + (attr.nl == \"\" and 0 or 1)\n"
"    s[index] = s[index] or \"\"\n"
"    s[index] = s[index] .. format(\"%s%s%s%s%s%s\", attr.nl, attr.indent,\n"
"                     attr.prefix, attr.prefix_sep, no_ASS(str), attr.suffix)\n"
"    return true\n"
"end\n"
"\n"
"\n"
"-- Format and append a property.\n"
"-- A property whose value is either \140nil\140 or empty (hereafter called \"invalid\")\n"
"-- is skipped and not appended.\n"
"-- Returns \140false\140 in case nothing was appended, otherwise \140true\140.\n"
"--\n"
"-- s      : Table containing strings.\n"
"-- prop   : The property to query and format (based on its OSD representation).\n"
"-- attr   : Optional table to overwrite certain (formatting) attributes for\n"
"--          this property.\n"
"-- exclude: Optional table containing keys which are considered invalid values\n"
"--          for this property. Specifying this will replace empty string as\n"
"--          default invalid value (nil is always invalid).\n"
"local function append_property(s, prop, attr, excluded)\n"
"    excluded = excluded or {[\"\"] = true}\n"
"    local ret = mp.get_property_osd(prop)\n"
"    if not ret or excluded[ret] then\n"
"        if o.debug then\n"
"            print(\"No value for property: \" .. prop)\n"
"        end\n"
"        return false\n"
"    end\n"
"    return append(s, ret, attr)\n"
"end\n"
"\n"
"local function sorted_keys(t, comp_fn)\n"
"    local keys = {}\n"
"    for k,_ in pairs(t) do\n"
"        keys[#keys+1] = k\n"
"    end\n"
"    table.sort(keys, comp_fn)\n"
"    return keys\n"
"end\n"
"\n"
"local function scroll_hint(search)\n"
"    local hint = format(\"(hint: scroll with %s/%s\", o.key_scroll_up, o.key_scroll_down)\n"
"    if search then\n"
"        hint = hint .. \" and search with \" .. o.key_search\n"
"    end\n"
"    hint = hint .. \")\"\n"
"    if not o.use_ass then return \" \" .. hint end\n"
"    return format(\" {\\\\fs%s}%s{\\\\fs%s}\", font_size * 0.66, hint, font_size)\n"
"end\n"
"\n"
"local function append_perfdata(header, s, dedicated_page)\n"
"    local vo_p = mp.get_property_native(\"vo-passes\")\n"
"    if not vo_p then\n"
"        return\n"
"    end\n"
"\n"
"    -- Sums of all last/avg/peak values\n"
"    local last_s, avg_s, peak_s = {}, {}, {}\n"
"    for frame, data in pairs(vo_p) do\n"
"        last_s[frame], avg_s[frame], peak_s[frame] = 0, 0, 0\n"
"        for _, pass in ipairs(data) do\n"
"            last_s[frame] = last_s[frame] + pass[\"last\"]\n"
"            avg_s[frame]  = avg_s[frame]  + pass[\"avg\"]\n"
"            peak_s[frame] = peak_s[frame] + pass[\"peak\"]\n"
"        end\n"
"    end\n"
"\n"
"    -- Pretty print measured time\n"
"    local function pp(i)\n"
"        -- rescale to microseconds for a saner display\n"
"        return format(\"%5d\", i / 1000)\n"
"    end\n"
"\n"
"    -- Format n/m with a font weight based on the ratio\n"
"    local function p(n, m)\n"
"        local i = 0\n"
"        if m > 0 then\n"
"            i = tonumber(n) / m\n"
"        end\n"
"        -- Calculate font weight. 100 is minimum, 400 is normal, 700 bold, 900 is max\n"
"        local w = (700 * math.sqrt(i)) + 200\n"
"        if not o.use_ass then\n"
"            local str = format(\"%3d%%\", i * 100)\n"
"            return w >= 700 and bold(str) or str\n"
"        end\n"
"        return format(\"{\\\\b%d}%3d%%{\\\\b0}\", w, i * 100)\n"
"    end\n"
"\n"
"    local font_small = o.use_ass and format(\"{\\\\fs%s}\", font_size * 0.66) or \"\"\n"
"    local font_normal = o.use_ass and format(\"{\\\\fs%s}\", font_size) or \"\"\n"
"    local font = o.use_ass and format(\"{\\\\fn%s}\", o.font) or \"\"\n"
"    local font_mono = o.use_ass and format(\"{\\\\fn%s}\", o.font_mono) or \"\"\n"
"    local indent = o.use_ass and \"\\\\h\" or \" \"\n"
"\n"
"    -- ensure that the fixed title is one element and every scrollable line is\n"
"    -- also one single element.\n"
"    local h = dedicated_page and header or s\n"
"    h[#h+1] = format(\"%s%s%s%s%s%s%s%s\",\n"
"                     dedicated_page and \"\" or o.nl, dedicated_page and \"\" or o.indent,\n"
"                     bold(\"Frame Timings:\"), o.prefix_sep, font_small,\n"
"                     \"(last/average/peak \316\274s)\", font_normal,\n"
"                     dedicated_page and scroll_hint() or \"\")\n"
"\n"
"    for _,frame in ipairs(sorted_keys(vo_p)) do  -- ensure fixed display order\n"
"        local data = vo_p[frame]\n"
"        local f = \"%s%s%s%s%s / %s / %s %s%s%s%s%s%s\"\n"
"\n"
"        if dedicated_page then\n"
"            s[#s+1] = format(\"%s%s%s:\", o.nl, o.indent,\n"
"                             bold(frame:gsub(\"^%l\", string.upper)))\n"
"\n"
"            for _, pass in ipairs(data) do\n"
"                s[#s+1] = format(f, o.nl, o.indent, o.indent,\n"
"                                 font_mono, pp(pass[\"last\"]),\n"
"                                 pp(pass[\"avg\"]), pp(pass[\"peak\"]),\n"
"                                 o.prefix_sep .. indent, p(pass[\"last\"], last_s[frame]),\n"
"                                 font, o.prefix_sep, o.prefix_sep, pass[\"desc\"])\n"
"\n"
"                if o.plot_perfdata and o.use_ass then\n"
"                    -- use the same line that was already started for this iteration\n"
"                    s[#s] = s[#s] ..\n"
"                              generate_graph(pass[\"samples\"], pass[\"count\"],\n"
"                                             pass[\"count\"], pass[\"peak\"],\n"
"                                             pass[\"avg\"], 0.9, 0.25)\n"
"                end\n"
"            end\n"
"\n"
"            -- Print sum of timing values as \"Total\"\n"
"            s[#s+1] = format(f, o.nl, o.indent, o.indent,\n"
"                             font_mono, pp(last_s[frame]),\n"
"                             pp(avg_s[frame]), pp(peak_s[frame]),\n"
"                             o.prefix_sep, bold(\"Total\"), font, \"\", \"\", \"\")\n"
"        else\n"
"            -- for the simplified view, we just print the sum of each pass\n"
"            s[#s+1] = format(f, o.nl, o.indent, o.indent, font_mono,\n"
"                            pp(last_s[frame]), pp(avg_s[frame]), pp(peak_s[frame]),\n"
"                            \"\", \"\", font, o.prefix_sep, o.prefix_sep,\n"
"                            frame:gsub(\"^%l\", string.upper))\n"
"        end\n"
"    end\n"
"end\n"
"\n"
"-- command prefix tokens to strip - includes generic property commands\n"
"local cmd_prefixes = {\n"
"    osd_auto=1, no_osd=1, osd_bar=1, osd_msg=1, osd_msg_bar=1, raw=1, sync=1,\n"
"    async=1, expand_properties=1, repeatable=1, nonrepeatable=1, set=1, add=1,\n"
"    multiply=1, toggle=1, cycle=1, cycle_values=1, [\"!reverse\"]=1, change_list=1,\n"
"}\n"
"-- commands/writable-properties prefix sub-words (followed by -) to strip\n"
"local name_prefixes = {\n"
"    define=1, delete=1, enable=1, disable=1, dump=1, write=1, drop=1, revert=1,\n"
"    ab=1, hr=1, secondary=1, current=1,\n"
"}\n"
"-- extract a command \"subject\" from a command string, by removing all\n"
"-- generic prefix tokens and then returning the first interesting sub-word\n"
"-- of the next token. For target-script name we also check another token.\n"
"-- The tokenizer works fine for things we care about - valid mpv commands,\n"
"-- properties and script names, possibly quoted, white-space[s]-separated.\n"
"-- It's decent in practice, and worst case is \"incorrect\" subject.\n"
"local function cmd_subject(cmd)\n"
"    cmd = cmd:gsub(\";.*\", \"\"):gsub(\"%-\", \"_\")  -- only first cmd, s/-/_/\n"
"    local TOKEN = '^%s*[\"\\']\077([%w_!]*)'  -- captures+ends before (maybe) final \"\n"
"    local tok, sname, subw\n"
"\n"
"    repeat tok, cmd = cmd:match(TOKEN .. '[\"\\']\077(.*)')\n"
"    until not cmd_prefixes[tok]\n"
"    -- tok is the 1st non-generic command/property name token, cmd is the rest\n"
"\n"
"    sname = tok == \"script_message_to\" and cmd:match(TOKEN)\n"
"         or tok == \"script_binding\" and cmd:match(TOKEN .. \"/\")\n"
"    if sname and sname ~= \"\" then\n"
"        return \"script: \" .. sname\n"
"    end\n"
"\n"
"    -- return the first sub-word of tok which is not a useless prefix\n"
"    repeat subw, tok = tok:match(\"([^_]*)_\077(.*)\")\n"
"    until tok == \"\" or not name_prefixes[subw]\n"
"    return subw:len() > 1 and subw or \"[unknown]\"\n"
"end\n"
"\n"
"-- key names are valid UTF-8, ascii7 except maybe the last/only codepoint.\n"
"-- we count codepoints and ignore wcwidth. no need for grapheme clusters.\n"
"-- our error for alignment is at most one cell (if last CP is double-width).\n"
"-- (if k was valid but arbitrary: we'd count all bytes <0x80 or >=0xc0)\n"
"local function keyname_cells(k)\n"
"    local klen = k:len()\n"
"    if klen > 1 and k:byte(klen) >= 0x80 then  -- last/only CP is not ascii7\n"
"        repeat klen = klen-1\n"
"        until klen == 1 or k:byte(klen) >= 0xc0  -- last CP begins at klen\n"
"    end\n"
"    return klen\n"
"end\n"
"\n"
"local function get_kbinfo_lines()\n"
"    -- active keys: only highest priority of each key, and not our (stats) keys\n"
"    local bindings = mp.get_property_native(\"input-bindings\", {})\n"
"    local active = {}  -- map: key-name -> bind-info\n"
"    for _, bind in pairs(bindings) do\n"
"        if bind.priority >= 0 and (\n"
"               not active[bind.key] or\n"
"               (active[bind.key].is_weak and not bind.is_weak) or\n"
"               (bind.is_weak == active[bind.key].is_weak and\n"
"                bind.priority > active[bind.key].priority)\n"
"           ) and not bind.cmd:find(\"script-binding stats/__forced_\", 1, true)\n"
"           and bind.section ~= \"input_forced_console\"\n"
"           and (\n"
"               searched_text == nil or\n"
"               (bind.key .. bind.cmd):lower():find(searched_text, 1, true)\n"
"           )\n"
"        then\n"
"            active[bind.key] = bind\n"
"        end\n"
"    end\n"
"\n"
"    -- make an array, find max key len, add sort keys (.subject/.mods[_count])\n"
"    local ordered = {}\n"
"    local kspaces = \"\"  -- as many spaces as the longest key name\n"
"    for _, bind in pairs(active) do\n"
"        bind.subject = cmd_subject(bind.cmd)\n"
"        if bind.subject ~= \"ignore\" then\n"
"            ordered[#ordered+1] = bind\n"
"            _,_, bind.mods = bind.key:find(\"(.*)%+.\")\n"
"            _, bind.mods_count = bind.key:gsub(\"%+.\", \"\")\n"
"            if bind.key:len() > kspaces:len() then\n"
"                kspaces = string.rep(\" \", bind.key:len())\n"
"            end\n"
"        end\n"
"    end\n"
"\n"
"    local function align_right(key)\n"
"        return kspaces:sub(keyname_cells(key)) .. key\n"
"    end\n"
"\n"
"    -- sort by: subject, mod(ifier)s count, mods, key-len, lowercase-key, key\n"
"    table.sort(ordered, function(a, b)\n"
"        if a.subject ~= b.subject then\n"
"            return a.subject < b.subject\n"
"        elseif a.mods_count ~= b.mods_count then\n"
"            return a.mods_count < b.mods_count\n"
"        elseif a.mods ~= b.mods then\n"
"            return a.mods < b.mods\n"
"        elseif a.key:len() ~= b.key:len() then\n"
"            return a.key:len() < b.key:len()\n"
"        elseif a.key:lower() ~= b.key:lower() then\n"
"            return a.key:lower() < b.key:lower()\n"
"        else\n"
"            return a.key > b.key  -- only case differs, lowercase first\n"
"        end\n"
"    end)\n"
"\n"
"    -- key/subject pre/post formatting for terminal/ass.\n"
"    -- key/subject alignment uses spaces (with mono font if ass)\n"
"    -- word-wrapping is disabled for ass, or cut at 79 for the terminal\n"
"    local LTR = string.char(0xE2, 0x80, 0x8E)  -- U+200E Left To Right mark\n"
"    local term = not o.use_ass\n"
"    local kpre = term and \"\" or format(\"{\\\\q2\\\\fn%s}%s\", o.font_mono, LTR)\n"
"    local kpost = term and \" \" or format(\" {\\\\fn%s}\", o.font)\n"
"    local spre = term and kspaces .. \"   \"\n"
"                       or format(\"{\\\\q2\\\\fn%s}%s   {\\\\fn%s}{\\\\fs%d\\\\u1}\",\n"
"                                 o.font_mono, kspaces, o.font, 1.3*font_size)\n"
"    local spost = term and \"\" or format(\"{\\\\u0\\\\fs%d}%s\", font_size, text_style())\n"
"\n"
"    -- create the display lines\n"
"    local info_lines = {}\n"
"    local subject = nil\n"
"    for _, bind in ipairs(ordered) do\n"
"        if bind.subject ~= subject then  -- new subject (title)\n"
"            subject = bind.subject\n"
"            append(info_lines, \"\", {})\n"
"            append(info_lines, \"\", { prefix = spre .. subject .. spost })\n"
"        end\n"
"        if bind.comment then\n"
"            bind.cmd = bind.cmd .. \"  # \" .. bind.comment\n"
"        end\n"
"        append(info_lines, bind.cmd, { prefix = kpre .. no_ASS(align_right(bind.key)) .. kpost })\n"
"    end\n"
"    return info_lines\n"
"end\n"
"\n"
"local function append_general_perfdata(s)\n"
"    for i, data in ipairs(mp.get_property_native(\"perf-info\") or {}) do\n"
"        append(s, data.text or data.value, {prefix=\"[\"..tostring(i)..\"] \"..data.name..\":\"})\n"
"\n"
"        if o.plot_perfdata and o.use_ass and data.value then\n"
"            local buf = perf_buffers[data.name]\n"
"            if not buf then\n"
"                buf = {0, pos = 1, len = 50, max = 0}\n"
"                perf_buffers[data.name] = buf\n"
"            end\n"
"            graph_add_value(buf, data.value)\n"
"            s[#s] = s[#s] .. generate_graph(buf, buf.pos, buf.len, buf.max, nil, 0.8, 1)\n"
"        end\n"
"    end\n"
"end\n"
"\n"
"local function append_display_sync(s)\n"
"    if not mp.get_property_bool(\"display-sync-active\", false) then\n"
"        return\n"
"    end\n"
"\n"
"    local vspeed = append_property(s, \"video-speed-correction\", {prefix=\"DS:\"})\n"
"    if vspeed then\n"
"        append_property(s, \"audio-speed-correction\",\n"
"                        {prefix=\"/\", nl=\"\", indent=\" \", prefix_sep=\" \", no_prefix_markup=true})\n"
"    else\n"
"        append_property(s, \"audio-speed-correction\",\n"
"                        {prefix=\"DS:\" .. o.prefix_sep .. \" - / \", prefix_sep=\"\"})\n"
"    end\n"
"\n"
"    append_property(s, \"mistimed-frame-count\", {prefix=\"Mistimed:\", nl=\"\",\n"
"                                                indent=o.prefix_sep .. o.prefix_sep})\n"
"    append_property(s, \"vo-delayed-frame-count\", {prefix=\"Delayed:\", nl=\"\",\n"
"                                                  indent=o.prefix_sep .. o.prefix_sep})\n"
"\n"
"    -- As we need to plot some graphs we print jitter and ratio on their own lines\n"
"    if not display_timer.oneshot and (o.plot_vsync_ratio or o.plot_vsync_jitter) and o.use_ass then\n"
"        local ratio_graph = \"\"\n"
"        local jitter_graph = \"\"\n"
"        if o.plot_vsync_ratio then\n"
"            ratio_graph = generate_graph(vsratio_buf, vsratio_buf.pos,\n"
"                                         vsratio_buf.len, vsratio_buf.max, nil, 0.8, 1)\n"
"        end\n"
"        if o.plot_vsync_jitter then\n"
"            jitter_graph = generate_graph(vsjitter_buf, vsjitter_buf.pos,\n"
"                                          vsjitter_buf.len, vsjitter_buf.max, nil, 0.8, 1)\n"
"        end\n"
"        append_property(s, \"vsync-ratio\", {prefix=\"VSync Ratio:\",\n"
"                                           suffix=o.prefix_sep .. ratio_graph})\n"
"        append_property(s, \"vsync-jitter\", {prefix=\"VSync Jitter:\",\n"
"                                            suffix=o.prefix_sep .. jitter_graph})\n"
"    else\n"
"        -- Since no graph is needed we can print ratio/jitter on the same line and save some space\n"
"        local vr = append_property(s, \"vsync-ratio\", {prefix=\"VSync Ratio:\"})\n"
"        append_property(s, \"vsync-jitter\", {prefix=\"VSync Jitter:\",\n"
"                            nl=vr and \"\" or o.nl,\n"
"                            indent=vr and o.prefix_sep .. o.prefix_sep})\n"
"    end\n"
"end\n"
"\n"
"\n"
"local function append_filters(s, prop, prefix)\n"
"    local length = 0\n"
"    local filters = {}\n"
"\n"
"    for _,f in ipairs(mp.get_property_native(prop, {})) do\n"
"        local n = f.name\n"
"        if f.enabled ~= nil and not f.enabled then\n"
"            n = n .. \" (disabled)\"\n"
"        end\n"
"\n"
"        if f.label ~= nil then\n"
"            n = \"\100\" .. f.label .. \": \" .. n\n"
"        end\n"
"\n"
"        local p = {}\n"
"        for _,key in ipairs(sorted_keys(f.params)) do\n"
"            p[#p+1] = key .. \"=\" .. f.params[key]\n"
"        end\n"
"        if #p > 0 then\n"
"            p = \" [\" .. table.concat(p, \" \") .. \"]\"\n"
"        else\n"
"            p = \"\"\n"
"        end\n"
"\n"
"        length = length + n:len() + p:len()\n"
"        filters[#filters+1] = no_ASS(n) .. it(no_ASS(p))\n"
"    end\n"
"\n"
"    if #filters > 0 then\n"
"        local ret\n"
"        if length < o.filter_params_max_length then\n"
"            ret = table.concat(filters, \", \")\n"
"        else\n"
"            local sep = o.nl .. o.indent .. o.indent\n"
"            ret = sep .. table.concat(filters, sep)\n"
"        end\n"
"        s[#s+1] = o.nl .. o.indent .. bold(prefix) .. o.prefix_sep .. ret\n"
"    end\n"
"end\n"
"\n"
"\n"
"local function add_header(s)\n"
"    s[#s+1] = text_style()\n"
"end\n"
"\n"
"\n"
"local function add_file(s, print_cache, print_tags)\n"
"    append(s, \"\", {prefix=\"File:\", nl=\"\", indent=\"\"})\n"
"    append_property(s, \"filename\", {prefix_sep=\"\", nl=\"\", indent=\"\"})\n"
"    if mp.get_property_osd(\"filename\") ~= mp.get_property_osd(\"media-title\") then\n"
"        append_property(s, \"media-title\", {prefix=\"Title:\"})\n"
"    end\n"
"\n"
"    if print_tags then\n"
"        local tags = mp.get_property_native(\"display-tags\")\n"
"        local tags_displayed = 0\n"
"        for _, tag in ipairs(tags) do\n"
"            local value = mp.get_property(\"metadata/by-key/\" .. tag)\n"
"            if tag ~= \"Title\" and tags_displayed < o.file_tag_max_count\n"
"               and value and value:len() < o.file_tag_max_length then\n"
"                append(s, value, {prefix=string.gsub(tag, \"_\", \" \") .. \":\"})\n"
"                tags_displayed = tags_displayed + 1\n"
"            end\n"
"        end\n"
"    end\n"
"\n"
"    local editions = mp.get_property_number(\"editions\")\n"
"    local edition = mp.get_property_number(\"current-edition\")\n"
"    local ed_cond = (edition and editions > 1)\n"
"    if ed_cond then\n"
"        append_property(s, \"edition-list/\" .. tostring(edition) .. \"/title\",\n"
"                       {prefix=\"Edition:\"})\n"
"        append_property(s, \"edition-list/count\",\n"
"                        {prefix=\"(\" .. tostring(edition + 1) .. \"/\", suffix=\")\", nl=\"\",\n"
"                         indent=\" \", prefix_sep=\" \", no_prefix_markup=true})\n"
"    end\n"
"\n"
"    local ch_index = mp.get_property_number(\"chapter\")\n"
"    if ch_index and ch_index >= 0 then\n"
"        append_property(s, \"chapter-list/\" .. tostring(ch_index) .. \"/title\", {prefix=\"Chapter:\",\n"
"                        nl=ed_cond and \"\" or o.nl})\n"
"        append_property(s, \"chapter-list/count\",\n"
"                        {prefix=\"(\" .. tostring(ch_index + 1) .. \" /\", suffix=\")\", nl=\"\",\n"
"                         indent=\" \", prefix_sep=\" \", no_prefix_markup=true})\n"
"    end\n"
"\n"
"    local fs = append_property(s, \"file-size\", {prefix=\"Size:\"})\n"
"    append_property(s, \"file-format\", {prefix=\"Format/Protocol:\",\n"
"                                       nl=fs and \"\" or o.nl,\n"
"                                       indent=fs and o.prefix_sep .. o.prefix_sep})\n"
"\n"
"    if not print_cache then\n"
"        return\n"
"    end\n"
"\n"
"    local demuxer_cache = mp.get_property_native(\"demuxer-cache-state\", {})\n"
"    if demuxer_cache[\"fw-bytes\"] then\n"
"        demuxer_cache = demuxer_cache[\"fw-bytes\"] -- returns bytes\n"
"    else\n"
"        demuxer_cache = 0\n"
"    end\n"
"    local demuxer_secs = mp.get_property_number(\"demuxer-cache-duration\", 0)\n"
"    if demuxer_cache + demuxer_secs > 0 then\n"
"        append(s, utils.format_bytes_humanized(demuxer_cache), {prefix=\"Total Cache:\"})\n"
"        append(s, format(\"%.1f\", demuxer_secs), {prefix=\"(\", suffix=\" sec)\", nl=\"\",\n"
"               no_prefix_markup=true, prefix_sep=\"\", indent=o.prefix_sep})\n"
"    end\n"
"end\n"
"\n"
"\n"
"local function crop_noop(w, h, r)\n"
"    return r[\"crop-x\"] == 0 and r[\"crop-y\"] == 0 and\n"
"           r[\"crop-w\"] == w and r[\"crop-h\"] == h\n"
"end\n"
"\n"
"\n"
"local function crop_equal(r, ro)\n"
"    return r[\"crop-x\"] == ro[\"crop-x\"] and r[\"crop-y\"] == ro[\"crop-y\"] and\n"
"           r[\"crop-w\"] == ro[\"crop-w\"] and r[\"crop-h\"] == ro[\"crop-h\"]\n"
"end\n"
"\n"
"\n"
"local function append_resolution(s, r, prefix, w_prop, h_prop, video_res)\n"
"    if not r then\n"
"        return\n"
"    end\n"
"    w_prop = w_prop or \"w\"\n"
"    h_prop = h_prop or \"h\"\n"
"    if append(s, r[w_prop], {prefix=prefix}) then\n"
"        append(s, r[h_prop], {prefix=\"x\", nl=\"\", indent=\" \", prefix_sep=\" \",\n"
"                           no_prefix_markup=true})\n"
"        if r[\"aspect\"] ~= nil and not video_res then\n"
"            append(s, format(\"%.2f:1\", r[\"aspect\"]), {prefix=\"\", nl=\"\", indent=\"\",\n"
"                                                      no_prefix_markup=true})\n"
"            append(s, r[\"aspect-name\"], {prefix=\"(\", suffix=\")\", nl=\"\", indent=\" \",\n"
"                                         prefix_sep=\"\", no_prefix_markup=true})\n"
"        end\n"
"        if r[\"sar\"] ~= nil and video_res then\n"
"            append(s, format(\"%.2f:1\", r[\"sar\"]), {prefix=\"\", nl=\"\", indent=\"\",\n"
"                                                      no_prefix_markup=true})\n"
"            append(s, r[\"sar-name\"], {prefix=\"(\", suffix=\")\", nl=\"\", indent=\" \",\n"
"                                         prefix_sep=\"\", no_prefix_markup=true})\n"
"        end\n"
"        if r[\"s\"] then\n"
"            append(s, format(\"%.2f\", r[\"s\"]), {prefix=\"(\", suffix=\"x)\", nl=\"\",\n"
"                                               indent=o.prefix_sep, prefix_sep=\"\",\n"
"                                               no_prefix_markup=true})\n"
"        end\n"
"        -- We can skip crop if it is the same as video decoded resolution\n"
"        if r[\"crop-w\"] and (not video_res or\n"
"                            not crop_noop(r[w_prop], r[h_prop], r)) then\n"
"            append(s, format(\"[x: %d, y: %d, w: %d, h: %d]\",\n"
"                            r[\"crop-x\"], r[\"crop-y\"], r[\"crop-w\"], r[\"crop-h\"]),\n"
"                            {prefix=\"\", nl=\"\", indent=\"\", no_prefix_markup=true})\n"
"        end\n"
"    end\n"
"end\n"
"\n"
"\n"
"local function pq_eotf(x)\n"
"    if not x then\n"
"        return x;\n"
"    end\n"
"\n"
"    local PQ_M1 = 2610.0 / 4096 * 1.0 / 4\n"
"    local PQ_M2 = 2523.0 / 4096 * 128\n"
"    local PQ_C1 = 3424.0 / 4096\n"
"    local PQ_C2 = 2413.0 / 4096 * 32\n"
"    local PQ_C3 = 2392.0 / 4096 * 32\n"
"\n"
"    x = x ^ (1.0 / PQ_M2)\n"
"    x = max(x - PQ_C1, 0.0) / (PQ_C2 - PQ_C3 * x)\n"
"    x = x ^ (1.0 / PQ_M1)\n"
"    x = x * 10000.0\n"
"\n"
"    return x\n"
"end\n"
"\n"
"\n"
"local function append_hdr(s, hdr, video_out)\n"
"    if not hdr then\n"
"        return\n"
"    end\n"
"\n"
"    local function should_show(val)\n"
"        return val and val ~= 203 and val > 0\n"
"    end\n"
"\n"
"    -- If we are printing video out parameters it is just display, not mastering\n"
"    local display_prefix = video_out and \"Display:\" or \"Mastering display:\"\n"
"\n"
"    local indent = \"\"\n"
"\n"
"    if should_show(hdr[\"max-cll\"]) or should_show(hdr[\"max-luma\"]) then\n"
"        append(s, \"\", {prefix=\"HDR10:\"})\n"
"        if hdr[\"min-luma\"] and should_show(hdr[\"max-luma\"]) then\n"
"            -- libplacebo uses close to zero values as \"defined zero\"\n"
"            hdr[\"min-luma\"] = hdr[\"min-luma\"] <= 1e-6 and 0 or hdr[\"min-luma\"]\n"
"            append(s, format(\"%.2g / %.0f\", hdr[\"min-luma\"], hdr[\"max-luma\"]),\n"
"                {prefix=display_prefix, suffix=\" cd/m\302\262\", nl=\"\", indent=indent})\n"
"            indent = o.prefix_sep .. o.prefix_sep\n"
"        end\n"
"        if should_show(hdr[\"max-cll\"]) then\n"
"            append(s, hdr[\"max-cll\"], {prefix=\"MaxCLL:\", suffix=\" cd/m\302\262\", nl=\"\",\n"
"                                       indent=indent})\n"
"            indent = o.prefix_sep .. o.prefix_sep\n"
"        end\n"
"        if hdr[\"max-fall\"] and hdr[\"max-fall\"] > 0 then\n"
"            append(s, hdr[\"max-fall\"], {prefix=\"MaxFALL:\", suffix=\" cd/m\302\262\", nl=\"\",\n"
"                                        indent=indent})\n"
"        end\n"
"    end\n"
"\n"
"    indent = o.prefix_sep .. o.prefix_sep\n"
"\n"
"    if hdr[\"scene-max-r\"] or hdr[\"scene-max-g\"] or\n"
"       hdr[\"scene-max-b\"] or hdr[\"scene-avg\"] then\n"
"        append(s, \"\", {prefix=\"HDR10+:\"})\n"
"        append(s, format(\"%.1f / %.1f / %.1f\", hdr[\"scene-max-r\"] or 0,\n"
"                         hdr[\"scene-max-g\"] or 0, hdr[\"scene-max-b\"] or 0),\n"
"               {prefix=\"MaxRGB:\", suffix=\" cd/m\302\262\", nl=\"\", indent=\"\"})\n"
"        append(s, format(\"%.1f\", hdr[\"scene-avg\"] or 0),\n"
"               {prefix=\"Avg:\", suffix=\" cd/m\302\262\", nl=\"\", indent=indent})\n"
"    end\n"
"\n"
"    if hdr[\"max-pq-y\"] and hdr[\"avg-pq-y\"] then\n"
"        append(s, \"\", {prefix=\"PQ(Y):\"})\n"
"        append(s, format(\"%.2f cd/m\302\262 (%.2f%% PQ)\", pq_eotf(hdr[\"max-pq-y\"]),\n"
"                         hdr[\"max-pq-y\"] * 100), {prefix=\"Max:\", nl=\"\",\n"
"                         indent=\"\"})\n"
"        append(s, format(\"%.2f cd/m\302\262 (%.2f%% PQ)\", pq_eotf(hdr[\"avg-pq-y\"]),\n"
"                         hdr[\"avg-pq-y\"] * 100), {prefix=\"Avg:\", nl=\"\",\n"
"                         indent=indent})\n"
"    end\n"
"end\n"
"\n"
"\n"
"local function append_img_params(s, r, ro)\n"
"    if not r then\n"
"        return\n"
"    end\n"
"\n"
"    append_resolution(s, r, \"Resolution:\", \"w\", \"h\", true)\n"
"    if ro and (r[\"w\"] ~= ro[\"dw\"] or r[\"h\"] ~= ro[\"dh\"]) then\n"
"        if ro[\"crop-w\"] and (crop_noop(r[\"w\"], r[\"h\"], ro) or crop_equal(r, ro)) then\n"
"            ro[\"crop-w\"] = nil\n"
"        end\n"
"        append_resolution(s, ro, \"Output Resolution:\", \"dw\", \"dh\")\n"
"    end\n"
"\n"
"    local indent = o.prefix_sep .. o.prefix_sep\n"
"    r = ro or r\n"
"\n"
"    local pixel_format = r[\"hw-pixelformat\"] or r[\"pixelformat\"]\n"
"    append(s, pixel_format, {prefix=\"Format:\"})\n"
"    append(s, r[\"colorlevels\"], {prefix=\"Levels:\", nl=\"\", indent=indent})\n"
"    if r[\"chroma-location\"] and r[\"chroma-location\"] ~= \"unknown\" then\n"
"        append(s, r[\"chroma-location\"], {prefix=\"Chroma Loc:\", nl=\"\", indent=indent})\n"
"    end\n"
"\n"
"    -- Group these together to save vertical space\n"
"    append(s, r[\"colormatrix\"], {prefix=\"Colormatrix:\"})\n"
"    append(s, r[\"primaries\"], {prefix=\"Primaries:\", nl=\"\", indent=indent})\n"
"    append(s, r[\"gamma\"], {prefix=\"Transfer:\", nl=\"\", indent=indent})\n"
"end\n"
"\n"
"\n"
"local function append_fps(s, prop, eprop)\n"
"    local fps = mp.get_property_osd(prop)\n"
"    local efps = mp.get_property_osd(eprop)\n"
"    local single = eprop == \"\" or (fps ~= \"\" and efps ~= \"\" and fps == efps)\n"
"    local unit = prop == \"display-fps\" and \" Hz\" or \" fps\"\n"
"    local suffix = single and \"\" or \" (specified)\"\n"
"    local esuffix = single and \"\" or \" (estimated)\"\n"
"    local prefix = prop == \"display-fps\" and \"Refresh Rate:\" or \"Frame Rate:\"\n"
"    local nl = o.nl\n"
"    local indent = o.indent\n"
"\n"
"    if fps ~= \"\" and append(s, fps, {prefix=prefix, suffix=unit .. suffix}) then\n"
"        prefix = \"\"\n"
"        nl = \"\"\n"
"        indent = \"\"\n"
"    end\n"
"\n"
"    if not single and efps ~= \"\" then\n"
"        append(s, efps,\n"
"               {prefix=prefix, suffix=unit .. esuffix, nl=nl, indent=indent})\n"
"    end\n"
"end\n"
"\n"
"\n"
"local function add_video_out(s)\n"
"    local vo = mp.get_property_native(\"current-vo\")\n"
"    if not vo then\n"
"        return\n"
"    end\n"
"\n"
"    append(s, \"\", {prefix=\"Display:\", nl=o.nl .. o.nl, indent=\"\"})\n"
"    append(s, vo, {prefix_sep=\"\", nl=\"\", indent=\"\"})\n"
"    append_property(s, \"display-names\", {prefix_sep=\"\", prefix=\"(\", suffix=\")\",\n"
"                                         no_prefix_markup=true, nl=\"\", indent=\" \"})\n"
"    append(s, mp.get_property_native(\"current-gpu-context\"),\n"
"           {prefix=\"Context:\", nl=\"\", indent=o.prefix_sep .. o.prefix_sep})\n"
"    append_property(s, \"avsync\", {prefix=\"A-V:\"})\n"
"    append_fps(s, \"display-fps\", \"estimated-display-fps\")\n"
"    if append_property(s, \"decoder-frame-drop-count\",\n"
"                       {prefix=\"Dropped Frames:\", suffix=\" (decoder)\"}) then\n"
"        append_property(s, \"frame-drop-count\", {suffix=\" (output)\", nl=\"\", indent=\"\"})\n"
"    end\n"
"    append_display_sync(s)\n"
"    append_perfdata(nil, s, false)\n"
"\n"
"    if mp.get_property_native(\"deinterlace-active\") then\n"
"        append_property(s, \"deinterlace\", {prefix=\"Deinterlacing:\"})\n"
"    end\n"
"\n"
"    local scale = nil\n"
"    if not mp.get_property_native(\"fullscreen\") then\n"
"        scale = mp.get_property_native(\"current-window-scale\")\n"
"    end\n"
"\n"
"    local od = mp.get_property_native(\"osd-dimensions\")\n"
"    local rt = mp.get_property_native(\"video-target-params\")\n"
"    local r = rt or {}\n"
"\n"
"    -- Add window scale\n"
"    r[\"s\"] = scale\n"
"    r[\"crop-x\"] = od[\"ml\"]\n"
"    r[\"crop-y\"] = od[\"mt\"]\n"
"    r[\"crop-w\"] = od[\"w\"] - od[\"ml\"] - od[\"mr\"]\n"
"    r[\"crop-h\"] = od[\"h\"] - od[\"mt\"] - od[\"mb\"]\n"
"\n"
"    if not rt then\n"
"        r[\"w\"] = r[\"crop-w\"]\n"
"        r[\"h\"] = r[\"crop-h\"]\n"
"        append_resolution(s, r, \"Resolution:\", \"w\", \"h\", true)\n"
"        return\n"
"    end\n"
"\n"
"    append_img_params(s, r)\n"
"    append_hdr(s, r, true)\n"
"end\n"
"\n"
"\n"
"local function add_video(s)\n"
"    local r = mp.get_property_native(\"video-params\")\n"
"    local ro = mp.get_property_native(\"video-out-params\")\n"
"    -- in case of e.g. lavfi-complex there can be no input video, only output\n"
"    if not r then\n"
"        r = ro\n"
"    end\n"
"    if not r then\n"
"        return\n"
"    end\n"
"\n"
"    local track = mp.get_property_native(\"current-tracks/video\")\n"
"    if track then\n"
"        append(s, \"\", {prefix=track.image and \"Image:\" or \"Video:\", nl=o.nl .. o.nl, indent=\"\"})\n"
"        append(s, track[\"codec-desc\"], {prefix_sep=\"\", nl=\"\", indent=\"\"})\n"
"        append(s, track[\"codec-profile\"], {prefix=\"[\", nl=\"\", indent=\" \", prefix_sep=\"\",\n"
"               no_prefix_markup=true, suffix=\"]\"})\n"
"        if track[\"codec\"] ~= track[\"decoder\"] then\n"
"            append(s, track[\"decoder\"], {prefix=\"[\", nl=\"\", indent=\" \", prefix_sep=\"\",\n"
"                   no_prefix_markup=true, suffix=\"]\"})\n"
"        end\n"
"        append_property(s, \"hwdec-current\", {prefix=\"HW:\", nl=\"\",\n"
"                        indent=o.prefix_sep .. o.prefix_sep,\n"
"                        no_prefix_markup=false, suffix=\"\"}, {no=true, [\"\"]=true})\n"
"    end\n"
"    local has_prefix = false\n"
"    if o.show_frame_info then\n"
"        if append_property(s, \"estimated-frame-number\", {prefix=\"Frame:\"}) then\n"
"            append_property(s, \"estimated-frame-count\", {indent=\" / \", nl=\"\",\n"
"                                                        prefix_sep=\"\"})\n"
"            has_prefix = true\n"
"        end\n"
"        local frame_info = mp.get_property_native(\"video-frame-info\")\n"
"        if frame_info and frame_info[\"picture-type\"] then\n"
"            local attrs = has_prefix and {prefix=\"(\", suffix=\")\", indent=\" \", nl=\"\",\n"
"                                          prefix_sep=\"\", no_prefix_markup=true}\n"
"                                      or {prefix=\"Picture Type:\"}\n"
"            append(s, frame_info[\"picture-type\"], attrs)\n"
"            has_prefix = true\n"
"        end\n"
"        if frame_info and frame_info[\"interlaced\"] then\n"
"            local attrs = has_prefix and {indent=\" \", nl=\"\", prefix_sep=\"\"}\n"
"                                      or {prefix=\"Picture Type:\"}\n"
"            append(s, \"Interlaced\", attrs)\n"
"        end\n"
"    end\n"
"\n"
"    if mp.get_property_native(\"current-tracks/video/image\") == false then\n"
"        append_fps(s, \"container-fps\", \"estimated-vf-fps\")\n"
"    end\n"
"    append_img_params(s, r, ro)\n"
"    append_hdr(s, ro)\n"
"    append_property(s, \"video-bitrate\", {prefix=\"Bitrate:\"})\n"
"    append_filters(s, \"vf\", \"Filters:\")\n"
"end\n"
"\n"
"\n"
"local function add_audio(s)\n"
"    local r = mp.get_property_native(\"audio-params\")\n"
"    -- in case of e.g. lavfi-complex there can be no input audio, only output\n"
"    local ro = mp.get_property_native(\"audio-out-params\") or r\n"
"    r = r or ro\n"
"    if not r then\n"
"        return\n"
"    end\n"
"\n"
"    local merge = function(rr, rro, prop)\n"
"        local a = rr[prop] or rro[prop]\n"
"        local b = rro[prop] or rr[prop]\n"
"        return (a == b or a == nil) and a or (a .. \" \342\236\234 \" .. b)\n"
"    end\n"
"\n"
"    append(s, \"\", {prefix=\"Audio:\", nl=o.nl .. o.nl, indent=\"\"})\n"
"    local track = mp.get_property_native(\"current-tracks/audio\")\n"
"    if track then\n"
"        append(s, track[\"codec-desc\"], {prefix_sep=\"\", nl=\"\", indent=\"\"})\n"
"        append(s, track[\"codec-profile\"], {prefix=\"[\", nl=\"\", indent=\" \", prefix_sep=\"\",\n"
"               no_prefix_markup=true, suffix=\"]\"})\n"
"        if track[\"codec\"] ~= track[\"decoder\"] then\n"
"            append(s, track[\"decoder\"], {prefix=\"[\", nl=\"\", indent=\" \", prefix_sep=\"\",\n"
"                   no_prefix_markup=true, suffix=\"]\"})\n"
"        end\n"
"    end\n"
"    append_property(s, \"current-ao\", {prefix=\"AO:\", nl=\"\",\n"
"                                      indent=o.prefix_sep .. o.prefix_sep})\n"
"    local dev = append_property(s, \"audio-device\", {prefix=\"Device:\"})\n"
"    local ao_mute = mp.get_property_native(\"ao-mute\") and \" (Muted)\" or \"\"\n"
"    append_property(s, \"ao-volume\", {prefix=\"AO Volume:\", suffix=\"%\" .. ao_mute,\n"
"                                     nl=dev and \"\" or o.nl,\n"
"                                     indent=dev and o.prefix_sep .. o.prefix_sep})\n"
"    if math.abs(mp.get_property_native(\"audio-delay\")) > 1e-6 then\n"
"        append_property(s, \"audio-delay\", {prefix=\"A-V delay:\"})\n"
"    end\n"
"    local cc = append(s, merge(r, ro, \"channel-count\"), {prefix=\"Channels:\"})\n"
"    append(s, merge(r, ro, \"format\"), {prefix=\"Format:\", nl=cc and \"\" or o.nl,\n"
"                            indent=cc and o.prefix_sep .. o.prefix_sep})\n"
"    append(s, merge(r, ro, \"samplerate\"), {prefix=\"Sample Rate:\", suffix=\" Hz\"})\n"
"    append_property(s, \"audio-bitrate\", {prefix=\"Bitrate:\"})\n"
"    append_filters(s, \"af\", \"Filters:\")\n"
"end\n"
"\n"
"\n"
"-- Determine whether ASS formatting shall/can be used and set formatting sequences\n"
"local function eval_ass_formatting()\n"
"    o.use_ass = o.ass_formatting and has_vo_window()\n"
"    if o.use_ass then\n"
"        o.nl = o.ass_nl\n"
"        o.indent = o.ass_indent\n"
"        o.prefix_sep = o.ass_prefix_sep\n"
"        o.b1 = o.ass_b1\n"
"        o.b0 = o.ass_b0\n"
"        o.it1 = o.ass_it1\n"
"        o.it0 = o.ass_it0\n"
"    else\n"
"        o.nl = o.no_ass_nl\n"
"        o.indent = o.no_ass_indent\n"
"        o.prefix_sep = o.no_ass_prefix_sep\n"
"        o.b1 = o.no_ass_b1\n"
"        o.b0 = o.no_ass_b0\n"
"        o.it1 = o.no_ass_it1\n"
"        o.it0 = o.no_ass_it0\n"
"    end\n"
"end\n"
"\n"
"-- assumptions:\n"
"--   s is composed of SGR escape sequences and/or printable UTF8 sequences\n"
"--   printable codepoints occupy one terminal cell (we don't have wcwidth)\n"
"--   tabstops are 8, 16, 24..., and the output starts at 0 or a tabstop\n"
"-- note: if maxwidth <= 2 and s doesn't fit: the result is \"...\" (more than 2)\n"
"local function term_ellipsis(s, maxwidth)\n"
"    local TAB, ESC, SGR_END = 9, 27, (\"m\"):byte()\n"
"    local width, ellipsis = 0, \"...\"\n"
"    local fit_len, in_sgr\n"
"\n"
"    for i = 1, #s do\n"
"        local x = s:byte(i)\n"
"\n"
"        if in_sgr then\n"
"            in_sgr = x ~= SGR_END\n"
"        elseif x == ESC then\n"
"            in_sgr = true\n"
"            ellipsis = \"\\27[0m...\"  -- ensure SGR reset\n"
"        elseif x < 128 or x >= 192 then  -- non UTF8-continuation\n"
"            -- tab adds till the next stop, else add 1\n"
"            width = width + (x == TAB and 8 - width % 8 or 1)\n"
"\n"
"            if fit_len == nil and width > maxwidth - 3 then\n"
"                fit_len = i - 1  -- adding \"...\" still fits maxwidth\n"
"            end\n"
"            if width > maxwidth then\n"
"                return s:sub(1, fit_len) .. ellipsis\n"
"            end\n"
"        end\n"
"    end\n"
"\n"
"    return s\n"
"end\n"
"\n"
"local function term_ellipsis_array(arr, from, to, max_width)\n"
"    for i = from, to do\n"
"        arr[i] = term_ellipsis(arr[i], max_width)\n"
"    end\n"
"    return arr\n"
"end\n"
"\n"
"-- split str into a table\n"
"-- example: local t = split(s, \"\\n\")\n"
"-- plain: whether pat is a plain string (default false - pat is a pattern)\n"
"local function split(str, pat, plain)\n"
"    local init = 1\n"
"    local r, i, find, sub = {}, 1, string.find, string.sub\n"
"    repeat\n"
"        local f0, f1 = find(str, pat, init, plain)\n"
"        r[i], i = sub(str, init, f0 and f0 - 1), i+1\n"
"        init = f0 and f1 + 1\n"
"    until f0 == nil\n"
"    return r\n"
"end\n"
"\n"
"-- Composes the output with header and scrollable content\n"
"-- Returns string of the finished page and the actually chosen offset\n"
"--\n"
"-- header      : table of the header where each entry is one line\n"
"-- content     : table of the content where each entry is one line\n"
"-- apply_scroll: scroll the content\n"
"local function finalize_page(header, content, apply_scroll)\n"
"    local term_size = mp.get_property_native(\"term-size\", {})\n"
"    local term_width = o.term_width_limit or term_size.w or 80\n"
"    local term_height = o.term_height_limit or term_size.h or 24\n"
"    local from, to = 1, #content\n"
"    if apply_scroll and term_height > 0 then\n"
"        -- Up to 40 lines for libass because it can put a big performance toll on\n"
"        -- libass to process many lines which end up outside (below) the screen.\n"
"        -- In the terminal reduce height by 2 for the status line (can be more then one line)\n"
"        local max_content_lines = (o.use_ass and 40 or term_height - 2) - #header\n"
"        -- in the terminal the scrolling should stop once the last line is visible\n"
"        local max_offset = o.use_ass and #content or #content - max_content_lines + 1\n"
"        from = max(1, min((pages[curr_page].offset or 1), max_offset))\n"
"        to = min(#content, from + max_content_lines - 1)\n"
"        pages[curr_page].offset = from\n"
"    end\n"
"    local output = table.concat(header) .. table.concat(content, \"\", from, to)\n"
"    if not o.use_ass and term_width > 0 then\n"
"        local t = split(output, \"\\n\", true)\n"
"        -- limit width for the terminal\n"
"        output = table.concat(term_ellipsis_array(t, 1, #t, term_width), \"\\n\")\n"
"    end\n"
"    return output, from\n"
"end\n"
"\n"
"-- Returns an ASS string with \"normal\" stats\n"
"local function default_stats()\n"
"    local stats = {}\n"
"    eval_ass_formatting()\n"
"    add_header(stats)\n"
"    add_file(stats, true, false)\n"
"    add_video_out(stats)\n"
"    add_video(stats)\n"
"    add_audio(stats)\n"
"    return finalize_page({}, stats, false)\n"
"end\n"
"\n"
"-- Returns an ASS string with extended VO stats\n"
"local function vo_stats()\n"
"    local header, content = {}, {}\n"
"    eval_ass_formatting()\n"
"    add_header(header)\n"
"    append_perfdata(header, content, true)\n"
"    header = {table.concat(header)}\n"
"    return finalize_page(header, content, true)\n"
"end\n"
"\n"
"local kbinfo_lines = nil\n"
"local function keybinding_info(after_scroll, bindlist)\n"
"    local header = {}\n"
"    local page = pages[o.key_page_4]\n"
"    eval_ass_formatting()\n"
"    add_header(header)\n"
"    append(header, \"\", {prefix=format(\"%s:%s\", page.desc, scroll_hint(true)), nl=\"\", indent=\"\"})\n"
"    header = {table.concat(header)}\n"
"\n"
"    if not kbinfo_lines or not after_scroll then\n"
"        kbinfo_lines = get_kbinfo_lines(o.term_width_limit)\n"
"    end\n"
"\n"
"    return finalize_page(header, kbinfo_lines, not bindlist)\n"
"end\n"
"\n"
"local function float2rational(x)\n"
"    local max_den = 100000\n"
"    local m00, m01, m10, m11 = 1, 0, 0, 1\n"
"    local a = math.floor(x)\n"
"    local frac = x - a\n"
"    while m10 * a + m11 <= max_den do\n"
"        local temp = m00 * a + m01\n"
"        m01 = m00\n"
"        m00 = temp\n"
"        temp = m10 * a + m11\n"
"        m11 = m10\n"
"        m10 = temp\n"
"\n"
"        if frac == 0 then\n"
"            break\n"
"        end\n"
"\n"
"        x = 1 / frac\n"
"        a = math.floor(x)\n"
"        frac = x - a\n"
"    end\n"
"    return m00, m10\n"
"end\n"
"\n"
"local function add_track(c, t, i)\n"
"    if not t then\n"
"        return\n"
"    end\n"
"\n"
"    local type = t.image and \"Image\" or t[\"type\"]:sub(1, 1):upper() .. t[\"type\"]:sub(2)\n"
"    append(c, \"\", {prefix=type .. \":\", nl=o.nl .. o.nl, indent=\"\"})\n"
"    append(c, t[\"title\"], {prefix_sep=\"\", nl=\"\", indent=\"\"})\n"
"    append(c, t[\"id\"], {prefix=\"ID:\"})\n"
"    append(c, t[\"src-id\"], {prefix=\"Demuxer ID:\", nl=\"\", indent=o.prefix_sep .. o.prefix_sep})\n"
"    append(c, t[\"program-id\"], {prefix=\"Program ID:\", nl=\"\", indent=o.prefix_sep .. o.prefix_sep})\n"
"    append(c, t[\"ff-index\"], {prefix=\"FFmpeg Index:\", nl=\"\", indent=o.prefix_sep .. o.prefix_sep})\n"
"    append(c, t[\"external-filename\"], {prefix=\"File:\"})\n"
"    append(c, \"\", {prefix=\"Flags:\"})\n"
"    local flags = {\"default\", \"forced\", \"dependent\", \"visual-impaired\",\n"
"                   \"hearing-impaired\", \"image\", \"albumart\", \"external\"}\n"
"    local any = false\n"
"    for _, flag in ipairs(flags) do\n"
"        if t[flag] then\n"
"            append(c, flag, {prefix=any and \", \" or \"\", nl=\"\", indent=\"\", prefix_sep=\"\"})\n"
"            any = true\n"
"        end\n"
"    end\n"
"    if not any then\n"
"        table.remove(c)\n"
"    end\n"
"    if append(c, t[\"codec-desc\"], {prefix=\"Format:\"}) then\n"
"        append(c, t[\"codec-profile\"], {prefix=\"[\", nl=\"\", indent=\" \", prefix_sep=\"\",\n"
"               no_prefix_markup=true, suffix=\"]\"})\n"
"        if t[\"codec\"] ~= t[\"decoder\"] then\n"
"            append(c, t[\"decoder\"], {prefix=\"[\", nl=\"\", indent=\" \", prefix_sep=\"\",\n"
"                   no_prefix_markup=true, suffix=\"]\"})\n"
"        end\n"
"    end\n"
"    append(c, t[\"lang\"], {prefix=\"Language:\"})\n"
"    append(c, t[\"demux-channel-count\"], {prefix=\"Channels:\"})\n"
"    append(c, t[\"demux-channels\"], {prefix=\"Channel Layout:\"})\n"
"    append(c, t[\"demux-samplerate\"], {prefix=\"Sample Rate:\", suffix=\" Hz\"})\n"
"    local function B(b) return b and string.format(\"%.2f\", b / 1024) end\n"
"    local bitrate = append(c, B(t[\"demux-bitrate\"]), {prefix=\"Bitrate:\", suffix=\" kbps\"})\n"
"    append(c, B(t[\"hls-bitrate\"]), {prefix=\"HLS Bitrate:\", suffix=\" kbps\",\n"
"                                    nl=bitrate and \"\" or o.nl,\n"
"                                    indent=bitrate and o.prefix_sep .. o.prefix_sep})\n"
"    append_resolution(c, {w=t[\"demux-w\"], h=t[\"demux-h\"], [\"crop-x\"]=t[\"demux-crop-x\"],\n"
"                          [\"crop-y\"]=t[\"demux-crop-y\"], [\"crop-w\"]=t[\"demux-crop-w\"],\n"
"                          [\"crop-h\"]=t[\"demux-crop-h\"]}, \"Resolution:\")\n"
"    if not t[\"image\"] and t[\"demux-fps\"] then\n"
"        append_fps(c, \"track-list/\" .. i .. \"/demux-fps\", \"\")\n"
"    end\n"
"    append(c, t[\"demux-rotation\"], {prefix=\"Rotation:\"})\n"
"    if t[\"demux-par\"] then\n"
"        local num, den = float2rational(t[\"demux-par\"])\n"
"        append(c, string.format(\"%d:%d\", num, den), {prefix=\"Pixel Aspect Ratio:\"})\n"
"    end\n"
"    local track_rg = t[\"replaygain-track-peak\"] ~= nil or t[\"replaygain-track-gain\"] ~= nil\n"
"    local album_rg = t[\"replaygain-album-peak\"] ~= nil or t[\"replaygain-album-gain\"] ~= nil\n"
"    if track_rg or album_rg then\n"
"        append(c, \"\", {prefix=\"Replay Gain:\"})\n"
"    end\n"
"    if track_rg then\n"
"        append(c, \"\", {prefix=\"Track:\", indent=o.indent .. o.prefix_sep, prefix_sep=\"\"})\n"
"        append(c, t[\"replaygain-track-gain\"], {prefix=\"Gain:\", suffix=\" dB\",\n"
"                                               nl=\"\", indent=o.prefix_sep})\n"
"        append(c, t[\"replaygain-track-peak\"], {prefix=\"Peak:\", suffix=\" dB\",\n"
"                                               nl=\"\", indent=o.prefix_sep})\n"
"    end\n"
"    if album_rg then\n"
"        append(c, \"\", {prefix=\"Album:\", indent=o.indent .. o.prefix_sep, prefix_sep=\"\"})\n"
"        append(c, t[\"replaygain-album-gain\"], {prefix=\"Gain:\", suffix=\" dB\",\n"
"                                               nl=\"\", indent=o.prefix_sep})\n"
"        append(c, t[\"replaygain-album-peak\"], {prefix=\"Peak:\", suffix=\" dB\",\n"
"                                               nl=\"\", indent=o.prefix_sep})\n"
"    end\n"
"    if t[\"dolby-vision-profile\"] or t[\"dolby-vision-level\"] then\n"
"        append(c, \"\", {prefix=\"Dolby Vision:\"})\n"
"        append(c, t[\"dolby-vision-profile\"], {prefix=\"Profile:\", nl=\"\", indent=\"\"})\n"
"        append(c, t[\"dolby-vision-level\"], {prefix=\"Level:\", nl=\"\",\n"
"                                            indent=t[\"dolby-vision-profile\"] and\n"
"                                            o.prefix_sep .. o.prefix_sep or \"\"})\n"
"    end\n"
"end\n"
"\n"
"local function track_info()\n"
"    local h, c = {}, {}\n"
"    eval_ass_formatting()\n"
"    add_header(h)\n"
"    local desc = pages[o.key_page_5].desc\n"
"    append(h, \"\", {prefix=format(\"%s:%s\", desc, scroll_hint()), nl=\"\", indent=\"\"})\n"
"    h = {table.concat(h)}\n"
"    table.insert(c, o.nl .. o.nl)\n"
"    add_file(c, false, true)\n"
"    for i, track in ipairs(mp.get_property_native(\"track-list\")) do\n"
"        if track['selected'] then\n"
"            add_track(c, track, i - 1)\n"
"        end\n"
"    end\n"
"    return finalize_page(h, c, true)\n"
"end\n"
"\n"
"local function perf_stats()\n"
"    local header, content = {}, {}\n"
"    eval_ass_formatting()\n"
"    add_header(header)\n"
"    local page = pages[o.key_page_0]\n"
"    append(header, \"\", {prefix=format(\"%s:%s\", page.desc, scroll_hint()), nl=\"\", indent=\"\"})\n"
"    append_general_perfdata(content)\n"
"    header = {table.concat(header)}\n"
"    return finalize_page(header, content, true)\n"
"end\n"
"\n"
"local function opt_time(t)\n"
"    if type(t) == type(1.1) then\n"
"        return mp.format_time(t)\n"
"    end\n"
"    return \"\077\"\n"
"end\n"
"\n"
"-- Returns an ASS string with stats about the demuxer cache etc.\n"
"local function cache_stats()\n"
"    local stats = {}\n"
"\n"
"    eval_ass_formatting()\n"
"    add_header(stats)\n"
"    append(stats, \"\", {prefix=\"Cache Info:\", nl=\"\", indent=\"\"})\n"
"\n"
"    local info = mp.get_property_native(\"demuxer-cache-state\")\n"
"    if info == nil then\n"
"        append(stats, \"Unavailable.\", {})\n"
"        return finalize_page({}, stats, false)\n"
"    end\n"
"\n"
"    local a = info[\"reader-pts\"]\n"
"    local b = info[\"cache-end\"]\n"
"\n"
"    append(stats, opt_time(a) .. \" - \" .. opt_time(b), {prefix = \"Packet Queue:\"})\n"
"\n"
"    local r = nil\n"
"    if a ~= nil and b ~= nil then\n"
"        r = b - a\n"
"    end\n"
"\n"
"    local r_graph = nil\n"
"    if not display_timer.oneshot and o.use_ass then\n"
"        r_graph = generate_graph(cache_ahead_buf, cache_ahead_buf.pos,\n"
"                                 cache_ahead_buf.len, cache_ahead_buf.max,\n"
"                                 nil, 0.8, 1)\n"
"        r_graph = o.prefix_sep .. r_graph\n"
"    end\n"
"    append(stats, opt_time(r), {prefix = \"Readahead:\", suffix = r_graph})\n"
"\n"
"    -- These states are not necessarily exclusive. They're about potentially\n"
"    -- separate mechanisms, whose states may be decoupled.\n"
"    local state = \"reading\"\n"
"    local seek_ts = info[\"debug-seeking\"]\n"
"    if seek_ts ~= nil then\n"
"        state = \"seeking (to \" .. mp.format_time(seek_ts) .. \")\"\n"
"    elseif info[\"eof\"] == true then\n"
"        state = \"eof\"\n"
"    elseif info[\"underrun\"] then\n"
"        state = \"underrun\"\n"
"    elseif info[\"idle\"]  == true then\n"
"        state = \"inactive\"\n"
"    end\n"
"    append(stats, state, {prefix = \"State:\"})\n"
"\n"
"    local speed = info[\"raw-input-rate\"] or 0\n"
"    local speed_graph = nil\n"
"    if not display_timer.oneshot and o.use_ass then\n"
"        speed_graph = generate_graph(cache_speed_buf, cache_speed_buf.pos,\n"
"                                     cache_speed_buf.len, cache_speed_buf.max,\n"
"                                     nil, 0.8, 1)\n"
"        speed_graph = o.prefix_sep .. speed_graph\n"
"    end\n"
"    append(stats, utils.format_bytes_humanized(speed) .. \"/s\", {prefix=\"Speed:\",\n"
"        suffix=speed_graph})\n"
"\n"
"    append(stats, utils.format_bytes_humanized(info[\"total-bytes\"]),\n"
"           {prefix = \"Total RAM:\"})\n"
"    append(stats, utils.format_bytes_humanized(info[\"fw-bytes\"]),\n"
"           {prefix = \"Forward RAM:\"})\n"
"\n"
"    local fc = info[\"file-cache-bytes\"]\n"
"    if fc ~= nil then\n"
"        fc = utils.format_bytes_humanized(fc)\n"
"    else\n"
"        fc = \"(disabled)\"\n"
"    end\n"
"    append(stats, fc, {prefix = \"Disk Cache:\"})\n"
"\n"
"    append(stats, info[\"debug-low-level-seeks\"], {prefix = \"Media Seeks:\"})\n"
"    append(stats, info[\"debug-byte-level-seeks\"], {prefix = \"Stream Seeks:\"})\n"
"\n"
"    append(stats, \"\", {prefix=\"Ranges:\", nl=o.nl .. o.nl, indent=\"\"})\n"
"\n"
"    append(stats, info[\"bof-cached\"] and \"yes\" or \"no\",\n"
"           {prefix = \"Start Cached:\"})\n"
"    append(stats, info[\"eof-cached\"] and \"yes\" or \"no\",\n"
"           {prefix = \"End Cached:\"})\n"
"\n"
"    local ranges = info[\"seekable-ranges\"] or {}\n"
"    for n, range in ipairs(ranges) do\n"
"        append(stats, mp.format_time(range[\"start\"]) .. \" - \" ..\n"
"                      mp.format_time(range[\"end\"]),\n"
"               {prefix = format(\"Range %s:\", n)})\n"
"    end\n"
"\n"
"    return finalize_page({}, stats, false)\n"
"end\n"
"\n"
"-- Record 1 sample of cache statistics.\n"
"-- (Unlike record_data(), this does not return a function, but runs directly.)\n"
"local function record_cache_stats()\n"
"    local info = mp.get_property_native(\"demuxer-cache-state\")\n"
"    if info == nil then\n"
"        return\n"
"    end\n"
"\n"
"    local a = info[\"reader-pts\"]\n"
"    local b = info[\"cache-end\"]\n"
"    if a ~= nil and b ~= nil then\n"
"        graph_add_value(cache_ahead_buf, b - a)\n"
"    end\n"
"\n"
"    graph_add_value(cache_speed_buf, info[\"raw-input-rate\"] or 0)\n"
"end\n"
"\n"
"cache_recorder_timer = mp.add_periodic_timer(0.25, record_cache_stats)\n"
"cache_recorder_timer:kill()\n"
"\n"
"-- Current page and <page key>:<page function> mapping\n"
"curr_page = o.key_page_1\n"
"pages = {\n"
"    [o.key_page_1] = { f = default_stats, desc = \"Default\" },\n"
"    [o.key_page_2] = { f = vo_stats, desc = \"Extended Frame Timings\", scroll = true },\n"
"    [o.key_page_3] = { f = cache_stats, desc = \"Cache Statistics\" },\n"
"    [o.key_page_4] = { f = keybinding_info, desc = \"Active Key Bindings\", scroll = true },\n"
"    [o.key_page_5] = { f = track_info, desc = \"Selected Tracks Info\", scroll = true },\n"
"    [o.key_page_0] = { f = perf_stats, desc = \"Internal Performance Info\", scroll = true },\n"
"}\n"
"\n"
"\n"
"-- Returns a function to record vsratio/jitter with the specified \140skip\140 value\n"
"local function record_data(skip)\n"
"    init_buffers()\n"
"    skip = max(skip, 0)\n"
"    local i = skip\n"
"    return function()\n"
"        if i < skip then\n"
"            i = i + 1\n"
"            return\n"
"        else\n"
"            i = 0\n"
"        end\n"
"\n"
"        if o.plot_vsync_jitter then\n"
"            local r = mp.get_property_number(\"vsync-jitter\")\n"
"            if r then\n"
"                vsjitter_buf.pos = (vsjitter_buf.pos % vsjitter_buf.len) + 1\n"
"                vsjitter_buf[vsjitter_buf.pos] = r\n"
"                vsjitter_buf.max = max(vsjitter_buf.max, r)\n"
"            end\n"
"        end\n"
"\n"
"        if o.plot_vsync_ratio then\n"
"            local r = mp.get_property_number(\"vsync-ratio\")\n"
"            if r then\n"
"                vsratio_buf.pos = (vsratio_buf.pos % vsratio_buf.len) + 1\n"
"                vsratio_buf[vsratio_buf.pos] = r\n"
"                vsratio_buf.max = max(vsratio_buf.max, r)\n"
"            end\n"
"        end\n"
"    end\n"
"end\n"
"\n"
"-- Call the function for \140page\140 and print it to OSD\n"
"local function print_page(page, after_scroll)\n"
"    -- the page functions assume we start in ass-enabled mode.\n"
"    -- that's true for mp.set_osd_ass, but not for mp.osd_message.\n"
"    local ass_content = pages[page].f(after_scroll)\n"
"    if o.persistent_overlay then\n"
"        mp.set_osd_ass(0, 0, ass_content)\n"
"    else\n"
"        mp.osd_message((o.use_ass and ass_start or \"\") .. ass_content,\n"
"                       display_timer.oneshot and o.duration or o.redraw_delay + 1)\n"
"    end\n"
"end\n"
"\n"
"local function update_scale(value)\n"
"    local scale_with_video\n"
"    if o.vidscale == \"auto\" then\n"
"        scale_with_video = mp.get_property_native(\"osd-scale-by-window\")\n"
"    else\n"
"        scale_with_video = o.vidscale == \"yes\"\n"
"    end\n"
"\n"
"    -- Calculate scaled metrics.\n"
"    local scale = 1\n"
"    if not scale_with_video and value > 0 then\n"
"        scale = 720 / value\n"
"    end\n"
"    font_size = o.font_size * scale\n"
"    border_size = o.border_size * scale\n"
"    shadow_x_offset = o.shadow_x_offset * scale\n"
"    shadow_y_offset = o.shadow_y_offset * scale\n"
"    plot_bg_border_width = o.plot_bg_border_width * scale\n"
"    if display_timer:is_enabled() then\n"
"        print_page(curr_page)\n"
"    end\n"
"end\n"
"\n"
"local function handle_osd_height_update(_, value)\n"
"    update_scale(value)\n"
"end\n"
"\n"
"local function handle_osd_scale_by_window_update()\n"
"    local value = mp.get_property_native(\"osd-height\")\n"
"    update_scale(value)\n"
"end\n"
"\n"
"local function clear_screen()\n"
"    if o.persistent_overlay then mp.set_osd_ass(0, 0, \"\") else mp.osd_message(\"\", 0) end\n"
"end\n"
"\n"
"local function scroll_delta(d)\n"
"    if display_timer.oneshot then display_timer:kill() ; display_timer:resume() end\n"
"    pages[curr_page].offset = (pages[curr_page].offset or 1) + d\n"
"    print_page(curr_page, true)\n"
"end\n"
"local function scroll_up() scroll_delta(-o.scroll_lines) end\n"
"local function scroll_down() scroll_delta(o.scroll_lines) end\n"
"\n"
"local function reset_scroll_offsets()\n"
"    for _, page in pairs(pages) do\n"
"        page.offset = nil\n"
"    end\n"
"end\n"
"local function bind_scroll()\n"
"    if not scroll_bound then\n"
"        mp.add_forced_key_binding(o.key_scroll_up, \"__forced_\" .. o.key_scroll_up,\n"
"                                  scroll_up, {repeatable=true})\n"
"        mp.add_forced_key_binding(o.key_scroll_down, \"__forced_\" .. o.key_scroll_down,\n"
"                                  scroll_down, {repeatable=true})\n"
"        scroll_bound = true\n"
"    end\n"
"end\n"
"local function unbind_scroll()\n"
"    if scroll_bound then\n"
"        mp.remove_key_binding(\"__forced_\"..o.key_scroll_up)\n"
"        mp.remove_key_binding(\"__forced_\"..o.key_scroll_down)\n"
"        scroll_bound = false\n"
"    end\n"
"end\n"
"\n"
"local function filter_bindings()\n"
"    input.get({\n"
"        prompt = \"Filter bindings:\",\n"
"        opened = function ()\n"
"            -- This is necessary to close the console if the oneshot\n"
"            -- display_timer expires without typing anything.\n"
"            searched_text = \"\"\n"
"        end,\n"
"        edited = function (text)\n"
"            reset_scroll_offsets()\n"
"            searched_text = text:lower()\n"
"            print_page(curr_page)\n"
"            if display_timer.oneshot then\n"
"                display_timer:kill()\n"
"                display_timer:resume()\n"
"            end\n"
"        end,\n"
"        submit = input.terminate,\n"
"        closed = function ()\n"
"            searched_text = nil\n"
"            if display_timer:is_enabled() then\n"
"                print_page(curr_page)\n"
"                if display_timer.oneshot then\n"
"                    display_timer:kill()\n"
"                    display_timer:resume()\n"
"                end\n"
"            end\n"
"        end,\n"
"    })\n"
"end\n"
"\n"
"local function bind_search()\n"
"    mp.add_forced_key_binding(o.key_search, \"__forced_\"..o.key_search, filter_bindings)\n"
"end\n"
"\n"
"local function unbind_search()\n"
"    mp.remove_key_binding(\"__forced_\"..o.key_search)\n"
"end\n"
"\n"
"local function update_scroll_bindings(k)\n"
"    if pages[k].scroll then\n"
"        bind_scroll()\n"
"    else\n"
"        unbind_scroll()\n"
"    end\n"
"\n"
"    if k == o.key_page_4 then\n"
"        bind_search()\n"
"    else\n"
"        unbind_search()\n"
"    end\n"
"end\n"
"\n"
"-- Add keybindings for every page\n"
"local function add_page_bindings()\n"
"    local function a(k)\n"
"        return function()\n"
"            reset_scroll_offsets()\n"
"            update_scroll_bindings(k)\n"
"            curr_page = k\n"
"            print_page(k)\n"
"            if display_timer.oneshot then display_timer:kill() ; display_timer:resume() end\n"
"        end\n"
"    end\n"
"    for k, _ in pairs(pages) do\n"
"        mp.add_forced_key_binding(k, \"__forced_\"..k, a(k), {repeatable=true})\n"
"    end\n"
"    update_scroll_bindings(curr_page)\n"
"end\n"
"\n"
"\n"
"-- Remove keybindings for every page\n"
"local function remove_page_bindings()\n"
"    for k, _ in pairs(pages) do\n"
"        mp.remove_key_binding(\"__forced_\"..k)\n"
"    end\n"
"    unbind_scroll()\n"
"    unbind_search()\n"
"end\n"
"\n"
"\n"
"local function process_key_binding(oneshot)\n"
"    reset_scroll_offsets()\n"
"    -- Stats are already being displayed\n"
"    if display_timer:is_enabled() then\n"
"        -- Previous and current keys were oneshot -> restart timer\n"
"        if display_timer.oneshot and oneshot then\n"
"            display_timer:kill()\n"
"            print_page(curr_page)\n"
"            display_timer:resume()\n"
"        -- Previous and current keys were toggling -> end toggling\n"
"        elseif not display_timer.oneshot and not oneshot then\n"
"            display_timer:kill()\n"
"            cache_recorder_timer:stop()\n"
"            if tm_viz_prev ~= nil then\n"
"                mp.set_property_native(\"tone-mapping-visualize\", tm_viz_prev)\n"
"                tm_viz_prev = nil\n"
"            end\n"
"            clear_screen()\n"
"            remove_page_bindings()\n"
"            if recorder then\n"
"                mp.unobserve_property(recorder)\n"
"                recorder = nil\n"
"            end\n"
"        end\n"
"    -- No stats are being displayed yet\n"
"    else\n"
"        if not oneshot and (o.plot_vsync_jitter or o.plot_vsync_ratio) then\n"
"            recorder = record_data(o.skip_frames)\n"
"            -- Rely on the fact that \"vsync-ratio\" is updated at the same time.\n"
"            -- Using \"none\" to get a sample any time, even if it does not change.\n"
"            -- Will stop working if \"vsync-jitter\" property change notification\n"
"            -- changes, but it's fine for an internal script.\n"
"            mp.observe_property(\"vsync-jitter\", \"none\", recorder)\n"
"        end\n"
"        if not oneshot and o.plot_tonemapping_lut then\n"
"            tm_viz_prev = mp.get_property_native(\"tone-mapping-visualize\")\n"
"            mp.set_property_native(\"tone-mapping-visualize\", true)\n"
"        end\n"
"        if not oneshot then\n"
"            cache_ahead_buf = {0, pos = 1, len = 50, max = 0}\n"
"            cache_speed_buf = {0, pos = 1, len = 50, max = 0}\n"
"            cache_recorder_timer:resume()\n"
"        end\n"
"        display_timer:kill()\n"
"        display_timer.oneshot = oneshot\n"
"        display_timer.timeout = oneshot and o.duration or o.redraw_delay\n"
"        add_page_bindings()\n"
"        print_page(curr_page)\n"
"        display_timer:resume()\n"
"    end\n"
"end\n"
"\n"
"\n"
"-- Create the timer used for redrawing (toggling) or clearing the screen (oneshot)\n"
"-- The duration here is not important and always set in process_key_binding()\n"
"display_timer = mp.add_periodic_timer(o.duration,\n"
"    function()\n"
"        if display_timer.oneshot then\n"
"            display_timer:kill() ; clear_screen() ; remove_page_bindings()\n"
"            -- Close the console only if it was opened for searching bindings.\n"
"            if searched_text then\n"
"                input.terminate()\n"
"            end\n"
"        else\n"
"            print_page(curr_page)\n"
"        end\n"
"    end)\n"
"display_timer:kill()\n"
"\n"
"-- Single invocation key binding\n"
"mp.add_key_binding(nil, \"display-stats\", function() process_key_binding(true) end,\n"
"    {repeatable=true})\n"
"\n"
"-- Toggling key binding\n"
"mp.add_key_binding(nil, \"display-stats-toggle\", function() process_key_binding(false) end,\n"
"    {repeatable=false})\n"
"\n"
"-- Single invocation bindings without key, can be used in input.conf to create\n"
"-- bindings for a specific page: \"e script-binding stats/display-page-2\"\n"
"for k, _ in pairs(pages) do\n"
"    mp.add_key_binding(nil, \"display-page-\" .. k,\n"
"        function()\n"
"            curr_page = k\n"
"            process_key_binding(true)\n"
"        end, {repeatable=true})\n"
"end\n"
"\n"
"-- Reprint stats immediately when VO was reconfigured, only when toggled\n"
"mp.register_event(\"video-reconfig\",\n"
"    function()\n"
"        if display_timer:is_enabled() then\n"
"            print_page(curr_page)\n"
"        end\n"
"    end)\n"
"\n"
"--  --script-opts=stats-bindlist=[-]{yes|<TERM-WIDTH>}\n"
"if o.bindlist ~= \"no\" then\n"
"    mp.command(\"no-osd set really-quiet yes\")\n"
"    if o.bindlist:sub(1, 1) == \"-\" then\n"
"        o.bindlist = o.bindlist:sub(2)\n"
"        o.no_ass_b0 = \"\"\n"
"        o.no_ass_b1 = \"\"\n"
"    end\n"
"    local width = max(40, math.floor(tonumber(o.bindlist) or 79))\n"
"    mp.add_timeout(0, function()  -- wait for all other scripts to finish init\n"
"        o.ass_formatting = false\n"
"        o.no_ass_indent = \" \"\n"
"        o.term_size = { w = width , h = 0}\n"
"        io.write(keybinding_info(false, true) .. \"\\n\")\n"
"        mp.command(\"quit\")\n"
"    end)\n"
"end\n"
"\n"
"mp.observe_property('osd-height', 'native', handle_osd_height_update)\n"
"mp.observe_property('osd-scale-by-window', 'native', handle_osd_scale_by_window_update)\n"
